@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@
 #include "pr_btspp.h"
 #include "sys_util.h"
 
-extern char 	    callerId[MAXLEN];
+extern char         callerId[MAXLEN];
 extern type_key*    repeatCmdPtr;
 extern char         tmp[MAXMAXLEN];
 
@@ -89,7 +89,7 @@ static string_t* substParams(const char* in, cmdParams *params)
 
             stringAppend(cmdBuffer, ptr);
             
-	    //DEBUG2("[EX]: cur cmd >%s<", cmdBuffer->str);
+            //DEBUG2("[EX]: cur cmd >%s<", cmdBuffer->str);
 
             break;
 
@@ -97,8 +97,8 @@ static string_t* substParams(const char* in, cmdParams *params)
 
             *papameterized = '\0';
 
-            stringAppend(cmdBuffer, ptr);	// First part
-            stringAppend(cmdBuffer, callerId);	// Add params
+            stringAppend(cmdBuffer, ptr);    // First part
+            stringAppend(cmdBuffer, callerId);    // Add params
 
             ptr = papameterized + pc + 2;
 
@@ -108,8 +108,8 @@ static string_t* substParams(const char* in, cmdParams *params)
 
             const char* m = getModeName();
 
-            stringAppend(cmdBuffer, ptr);	// First part
-            stringAppend(cmdBuffer, m);		// Add params
+            stringAppend(cmdBuffer, ptr);    // First part
+            stringAppend(cmdBuffer, m);        // Add params
 
             ptr = papameterized + pm + 2;
 
@@ -120,16 +120,16 @@ static string_t* substParams(const char* in, cmdParams *params)
 
             *papameterized = '\0';
 
-            stringAppend(cmdBuffer, ptr);	// First part
+            stringAppend(cmdBuffer, ptr);    // First part
             if (params == NULL) {   // value can be empty
                 logger(L_WARN, "[EX]: No parameters supplied!");
                 
-		// this is useless
+                // this is useless
                 //stringAppend(cmdBuffer, PARAM_START);
                 //stringAppend(cmdBuffer, PARAM_PARAM);
            
-	    } else {
-                stringAppend(cmdBuffer, params->value);	// Add params
+            } else {
+                stringAppend(cmdBuffer, params->value);    // Add params
             }
             ptr = papameterized + pp + 2;
 
@@ -139,7 +139,7 @@ static string_t* substParams(const char* in, cmdParams *params)
 
             *papameterized = '\0';
 
-            stringAppend(cmdBuffer, ptr);	// First part
+            stringAppend(cmdBuffer, ptr);    // First part
             if (params == NULL) {
                 logger(L_WARN, "[EX]: No parameters supplied !");
 
@@ -147,7 +147,7 @@ static string_t* substParams(const char* in, cmdParams *params)
                 //stringAppend(cmdBuffer, PARAM_START);
                 //stringAppend(cmdBuffer, PARAM_INDEX);
             } else {
-                stringAppend(cmdBuffer, params->index);	// Add params
+                stringAppend(cmdBuffer, params->index);    // Add params
             }
             ptr = papameterized + pi + 2;
 
@@ -179,7 +179,7 @@ static string_t* substParams(const char* in, cmdParams *params)
             varName[pLen] = '\0';
 
             *papameterized = '\0';
-            stringAppend(cmdBuffer, ptr);	    // First part
+            stringAppend(cmdBuffer, ptr);        // First part
 
             varData* var = searchVar(varName);
 
@@ -188,18 +188,18 @@ static string_t* substParams(const char* in, cmdParams *params)
                 //logger(L_DBG, "[EX]: substitute: searchVar() OK");
 
                 if (var->size > 0) {
-                    stringAppendLen(cmdBuffer, var->value, var->size);	// Add params
+                    stringAppendLen(cmdBuffer, var->value, var->size);    // Add params
                 //} else {
                 //    logger(L_DBG, "[EX]: substitute: empty var");
                 }
 
-                DEBUG2("[EX]: cmd var value >%s<", cmdBuffer->str);
+                //DEBUG2("[EX]: cmd var value >%s<", cmdBuffer->str);
 
-            // seems it is more confusing than helpful, but some sh scripting uses $()
-	    } else {    // Just copy
+        // seems it is more confusing than helpful, but some sh scripting uses $()
+        } else {    // Just copy
 
-                sprintf(tmp, "[EX]: Just copy >%s<", ptr);
-                logger(L_DBG,tmp);
+                //sprintf(tmp, "[EX]: Just copy >%s<", ptr);
+                //logger(L_DBG,tmp);
 
                 stringAppend(cmdBuffer, "$(");
                 stringAppend(cmdBuffer, varName);
@@ -209,7 +209,7 @@ static string_t* substParams(const char* in, cmdParams *params)
             free(varName);
         }
     }
-    //DEBUG2("[EX]: substParams: final cmd >%s<", cmdBuffer->str);
+    DEBUG2("[EX]: substParams: final cmd >%s<", cmdBuffer->str);
 
     free(command);
 
@@ -381,20 +381,23 @@ int handleHook(int hook)
 /*static void uploadPixOld(char *cmdLine)
 {
     DEBUG2("uploadPix >%s<", cmdLine);
-    int prefixSz = 0;          		// image,icon,_name_,_file_|image,window,_file_|cover, _file_
+    int prefixSz = 0;                  // image,icon,_name_,_file_|image,window,_file_|cover, _file_
 
     if (strstr(cmdLine,UPLOAD_PIX)   != NULL) {
         prefixSz = strlen(UPLOAD_PIX) + 1;
-    } else if (strstr(cmdLine,UPLOAD_COVER) != NULL) {
-        prefixSz = strlen(UPLOAD_COVER) + 1;
+    } else if (strstr(cmdLine,UPLOAD_COVER_DATA) != NULL) {
+        prefixSz = strlen(UPLOAD_COVER_DATA) + 1;
     } else {
         int nCh = strlen(UPLOAD_ICON) + 1;
         prefixSz = nCh + strstr(cmdLine+nCh,",") - (cmdLine+nCh) + 1;
+    } else {
+        int nCh = strlen(UPLOAD_COVER) + 1;
+        prefixSz = nCh + strstr(cmdLine+nCh,",") - (cmdLine+nCh) + 1;
     }
 
     char* fname = cmdLine + prefixSz;
     char* p = fname;
-    while (*p != '\0') {	// remove trailing \n if exists
+    while (*p != '\0') {    // remove trailing \n if exists
         if (*p == '\n') {
             *p = '\0';
             break;
@@ -404,7 +407,7 @@ int handleHook(int hook)
     //sprintf(tmp, "uploadPix file >%s<", fname);
     //logger(L_INF,tmp);
 
-    if (strstr(cmdLine,UPLOAD_COVER) != NULL &&
+    if (strstr(cmdLine,UPLOAD_COVER_DATA) != NULL &&
             isDataNew(ID_COVER, fname, strlen(fname)) != RC_OK) {
 
         logger(L_INF,"Skip upload same cover");
@@ -417,7 +420,7 @@ int handleHook(int hook)
     long fLen = 0;
     if(stat(fname, &buf) == -1) {
         logger(L_ERR,"can't get file size!");
-        if (strstr(cmdLine,UPLOAD_COVER) == NULL) {	// in case UPLOAD_COVER we will send empty image to clear previous one
+        if (strstr(cmdLine,UPLOAD_COVER_DATA) == NULL) {    // in case UPLOAD_COVER_DATA we will send empty image to clear previous one
             return;
         } else {
             logger(L_DBG,"uploadPix(): send empty cover");
@@ -452,7 +455,7 @@ int handleHook(int hook)
     uint32_t szh32 = (uint32_t) fLen;
     uint32_t szi32 = htonl(szh32);
 
-    char * fBuffer = (char*) calloc(fLen+prefixSz+12,1);	// 11 =  "Set("     ,<size of bin data=4bytes>    ");\0"
+    char * fBuffer = (char*) calloc(fLen+prefixSz+12,1);    // 11 =  "Set("     ,<size of bin data=4bytes>    ");\0"
     if (fBuffer == NULL) {
         logger(L_ERR,"no more memory!");
         return;
@@ -460,7 +463,7 @@ int handleHook(int hook)
 
     strcpy(fBuffer, "Set(");
     strncat(fBuffer, cmdLine, prefixSz+1);
-    memcpy((void*)fBuffer+prefixSz+4, (const void *) &szi32, 4);	// length on binary data
+    memcpy((void*)fBuffer+prefixSz+4, (const void *) &szi32, 4);    // length on binary data
 
     if (fp) {
         #ifdef __cplusplus
@@ -475,22 +478,25 @@ int handleHook(int hook)
     dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
     dm->type  = DM_SET;
     dm->size  = prefixSz+fLen+10;
-    dm->value = (void*) fBuffer;	// transfer ovnership to dispatcher
+    dm->value = (void*) fBuffer;    // transfer ovnership to dispatcher
 
     sendToDispatcher(dm);
 
     return;
 }*/
 
-static void uploadPix(char *cmdLine)
+void uploadPix(char *cmdLine)
 {
     DEBUG2("[EX]: uploadPix >%s<", cmdLine);
-    int prefixSz = 0;          		// image,icon,_name_,_file_|image,window,_file_|cover, _file_
+    int prefixSz = 0;                  // image,icon,_name_,_file_|image,window,_file_|cover,noname, _file_
 
     if (strstr(cmdLine,UPLOAD_PIX)   != NULL) {
         prefixSz = strlen(UPLOAD_PIX) + 1;
+    } else if (strstr(cmdLine,UPLOAD_COVER_DATA) != NULL) {
+        prefixSz = strlen(UPLOAD_COVER_DATA) + 1;
     } else if (strstr(cmdLine,UPLOAD_COVER) != NULL) {
-        prefixSz = strlen(UPLOAD_COVER) + 1;
+        int nCh = strlen(UPLOAD_COVER) + 1;
+        prefixSz = nCh + strstr(cmdLine+nCh,",") - (cmdLine+nCh) + 1;
     } else {
         int nCh = strlen(UPLOAD_ICON) + 1;
         prefixSz = nCh + strstr(cmdLine+nCh,",") - (cmdLine+nCh) + 1;
@@ -498,7 +504,7 @@ static void uploadPix(char *cmdLine)
 
     char* fname = cmdLine + prefixSz;
     char* p = fname;
-    while (*p != '\0') {	// remove trailing \n if exists
+    while (*p != '\0') {    // remove trailing \n if exists
         if (*p == '\n') {
             *p = '\0';
             break;
@@ -508,7 +514,7 @@ static void uploadPix(char *cmdLine)
     sprintf(tmp, "[EX]: uploadPix file >%s<", fname);
     logger(L_INF,tmp);
 
-    if (strstr(cmdLine,UPLOAD_COVER) != NULL && isDataNew(ID_SET_COVER, fname, strlen(fname)) != RC_OK) {
+    if (strstr(cmdLine,UPLOAD_COVER_DATA) != NULL && isDataNew(ID_SET_COVER, fname, strlen(fname)) != RC_OK) {
         logger(L_INF,"[EX]: Skip upload same cover");
         return;
     }
@@ -531,7 +537,7 @@ static void uploadPix(char *cmdLine)
     dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
     dm->type  = DM_SETFILE;
     dm->size  = -1;
-    dm->value = (void*) fBuffer;	// transfer ovnership to dispatcher
+    dm->value = (void*) fBuffer;    // transfer ovnership to dispatcher
     dm->file  = (void*) fName;
 
     sendToDispatcher(dm);
@@ -594,8 +600,8 @@ static int getCmd(int subtype, const char *descr, const char *exec, cmdParams* p
     if (isServerMode()) {
         dm->type  = DM_SET;
     } else {
-        free(dm);
         free(dm->value);
+        free(dm);
         return  EXIT_NOK;
     }
     
@@ -625,15 +631,20 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
     char *res = executeSimpleCommand(exec, &resLen);
     
     if (res == NULL) {
-	// just replace NULL to empty string
-	res = malloc(1);
-	*res = '\0';
+        // just replace NULL to empty string
+        res = malloc(1);
+        *res = '\0';
     }
 
     int dlen = strlen(descr);
 
-    if (stringHasPrefix(descr,UPLOAD_ICON) || stringHasPrefix(descr,UPLOAD_PIX) || subtype == ID_SET_COVER) {
-
+    if (stringHasPrefix(descr,UPLOAD_ICON)  || 
+        stringHasPrefix(descr,UPLOAD_PIX)   ||
+        stringHasPrefix(descr,UPLOAD_COVER) ||
+        (subtype == ID_SET_COVER && strstr(descr,",noname,") != NULL)) {
+        
+        INFO2("[EX]: ExecAndSet upload icon/image/cover of size %d",(int)resLen);
+       
         char *chunk = (char*) calloc(dlen + resLen + 2, 1);
 
         strcpy(chunk,descr);
@@ -643,6 +654,25 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
         uploadPix(chunk);
 
         free(chunk);
+   
+    } else if (subtype == ID_SET_COVER && strstr(descr,",by_name,") != NULL) {
+         
+        string_t* command = stringNew(CMD_SET);
+        stringAppend(command,"(");
+        stringAppend(command,descr);
+        stringAppend(command,",");
+        stringAppendLen(command,res,resLen);
+        stringAppend(command,");");
+
+        dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
+        dm->type  = DM_SET;
+        dm->size  = command->len;
+        dm->value = (void*) strdup(command->str);
+        dm->file  = NULL;
+        
+        stringFree(command,BOOL_YES);
+
+        sendToDispatcher(dm);
 
     } else {
 
@@ -653,7 +683,7 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
 
             // Replace all ");" inside buffer to "@@" since Java Client will separate commands by ");"
             char *p;
-            while ((p = strstr(res,");")) != NULL) {	    // binary data ???
+            while ((p = strstr(res,");")) != NULL) {        // binary data ???
                 *p = '@';
                 p++;
                 *p = '@';
@@ -665,7 +695,7 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
             char* decodedVal = res;
             if (isConvertable(subtype) && needConvert() == 1) {
                 decodedVal = convCharset(res, resLen, CNV_TO);
-		converted = BOOL_YES;
+                converted = BOOL_YES;
                 DEBUG2("[EX]: Decoded string >%s< ", (decodedVal ? decodedVal : "NULL"));
             }
             int sz = (decodedVal ? strlen(decodedVal) : 0);
@@ -691,9 +721,9 @@ static int execAndSetCmd(int subtype, const char *descr, const char *exec, cmdPa
             memcpy(chunk+5+dlen,decodedVal,sz);      // could be binary data here ????
             strcpy(chunk+5+dlen+sz,");");
             
-	    if (converted == BOOL_YES) {
+            if (converted == BOOL_YES) {
                 free(decodedVal);
-	    }
+            }
 
             #else
 
@@ -770,45 +800,48 @@ static int setCmd(int subtype, const char *descr, const char *exec, cmdParams* p
     if (descr != NULL && isServerMode() == EXIT_OK) {
        
 
-        if (strstr(descr,UPLOAD_ICON)  == descr ||	    // handle Set(cover|image,icon|window,[...]_image_file_name_); separately.
+        if (strstr(descr,UPLOAD_ICON)  == descr ||        // handle Set(cover|image,icon|window|cover,[...]_image_file_name_); separately.
             strstr(descr,UPLOAD_PIX)   == descr ||
-            subtype == ID_SET_COVER) {
+            strstr(descr,UPLOAD_COVER) == descr) {
 
             uploadPix((char*)descr);
+            
+        } else if (subtype == ID_SET_COVER) {
+        
+            if (strstr(descr,",noname,") != NULL) {
+            
+                uploadPix((char*)descr);
+                
+            } else if (strstr(descr,",by_name,") != NULL ||
+                       strstr(descr,",clear") != NULL) {
+            
+                string_t* command = stringNew(CMD_SET);
+                stringAppend(command,"(");
+                stringAppend(command,descr);
+                stringAppend(command,");");
+
+                dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
+                dm->type  = DM_SET;
+                dm->size  = command->len;
+                dm->value = (void*) strdup(command->str);
+                dm->file  = NULL;
+
+                stringFree(command,BOOL_YES);
+                
+                sendToDispatcher(dm);
+            }
 
         } else {
 
-            
-            // special case for Set(list,..,directory, )
-	    string_t*  dirCmd = NULL;
-
-	    /*
-	    if (subtype == ID_SET_LIST || subtype == ID_SET_ILIST) {
-	        
-		char * d = strstr(descr,",directory,");
-		
-        	if (d) {
-		    dirCmd = stringNew("");
-		    stringAppendLen(dirCmd,descr,d-descr+1);
-		    INFO2("[EX]: Command: Set dirlist %s", dirCmd->str);
-		    
-        	    d = (d + strlen(",directory,"));
-		    string_t* data = executeDirListCommand(subtype, d);
-		    if (data) {
-	        	stringAppend(dirCmd,data->str);
-			stringFree(data,BOOL_YES);
-		    }
-        	}
-	    }*/
-            const char* data = (dirCmd ? dirCmd->str : descr);
+            const char* data = descr;
 
             if (isDataNew(subtype, data, strlen(data)) != RC_OK) {
                 logger(L_INF, "[EX]: Skip to send the same data");
             } else {
                 logger(L_INF, "[EX]: Data is new");
-		
-        	string_t* command = stringNew(CMD_SET);
-		stringAppend(command,"(");
+        
+                string_t* command = stringNew(CMD_SET);
+                stringAppend(command,"(");
 
                 // Change \n in input to real '\n'
                 const char *p1 = data;
@@ -818,15 +851,15 @@ static int setCmd(int subtype, const char *descr, const char *exec, cmdParams* p
                     p2 = strstr(p1, "\\n");
 
                     if (p2 == NULL) {
- 			stringAppend(command,p1);
+                         stringAppend(command,p1);
                     } else {
- 			stringAppendLen(command,p1, p2-p1);
- 			stringAppend(command,"\n");
+                         stringAppendLen(command,p1, p2-p1);
+                         stringAppend(command,"\n");
                         p1 = p2 + 2;
                     }
-                 }
+                }
 
- 		stringAppend(command,");");
+                 stringAppend(command,");");
 
                 dMessage* dm = (dMessage*) malloc(sizeof(dMessage));
                 dm->type  = DM_SET;
@@ -835,13 +868,9 @@ static int setCmd(int subtype, const char *descr, const char *exec, cmdParams* p
                 dm->file  = NULL;
 
                 sendToDispatcher(dm);
-		
-		stringFree(command,BOOL_YES);
+        
+                stringFree(command,BOOL_YES);
             }
-	    
-	    if (dirCmd) {
-	        stringFree(dirCmd,BOOL_YES);
-	    }
         }
     }
 
@@ -874,7 +903,7 @@ static int makeVarCmd(const char *descr, const char *exec)
         if (resLen > 0 && *(res+resLen-1) == '\n') {
             resLen--;
         }
-    } else {	    // set "by_value"
+    } else {        // set "by_value"
         if (eValue) {
             resLen = strlen(eValue);
             res    = strdup(eValue);
@@ -1071,16 +1100,16 @@ int macroCmd(int subtype, const char *descr, const char *exec, cmdParams* params
             } else {
 
                 res = executeSimpleCommand(eValue, &resLen);
-		
+        
                 if (res == NULL) {
-		    logger(L_ERR, "[EX]: macroCmd: empty condition value, skip execution");
+            logger(L_ERR, "[EX]: macroCmd: empty condition value, skip execution");
                     return EXIT_NOK;
                 }
                 if (*(res+resLen-1) == '\n') {
                     resLen--;
                 }
             }
-        } else {	// set "by_value"
+        } else {    // set "by_value"
             if (eValue) {
                 resLen = strlen(eValue);
                 res    = strdup(eValue);
@@ -1093,11 +1122,11 @@ int macroCmd(int subtype, const char *descr, const char *exec, cmdParams* params
         if (res == NULL) {
             runIt = 0;
         } else {
-            if (strcmp(res,"0") != 0 && strcmp(res,"0\n") != 0) {	// test return code $?
+            if (strcmp(res,"0") != 0 && strcmp(res,"0\n") != 0) {    // test return code $?
                 runIt = 0;
             }
         }
-	free(res);
+    free(res);
     }
 
     if (runIt == 0) {
@@ -1215,7 +1244,7 @@ static int processCommands(SingleList* commands, cmdParams* p)
     SingleList* list = commands;
     while (list) {
 
-	cmdItem * item = (cmdItem *) list->data;
+    cmdItem * item = (cmdItem *) list->data;
 
         /*if (item->descr == NULL) {
                     sprintf(tmp, "Process next command NULL parameters");
@@ -1225,11 +1254,11 @@ static int processCommands(SingleList* commands, cmdParams* p)
             logger(L_DBG, tmp);
             */
         int ret = processOneCommand(item, p);
-	
+    
         if (ret != EXIT_OK) {
             if (ret == EXIT_STOP) {
                 logger(L_DBG, "[EX]: Stop execution on current command sequence");
-                return EXIT_STOP; 	// just stop execution of command sequence
+                return EXIT_STOP;     // just stop execution of command sequence
             }
         }
 
