@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -37,6 +37,7 @@
 #include "timer.h"
 #include "var.h"
 #include "peer.h"
+#include "state.h"
 
 extern int   getClip 	(char *string); // from atsend.h
 
@@ -350,6 +351,8 @@ static void handleExit()
     freeCfg();
     freeRegexps();
     dbusFinish();
+    
+    freeState();
 
     //printf("[EX]: handleExit EXIT\n");
 }
@@ -813,14 +816,17 @@ static int isInpLircCommand(char *cmdIn)
     }
 
     token = strtok_r(NULL," ",&bufPtr);
-    if (token && strncmp(token, "KEY_",4)==0) {
-        data = token + 4;
+    if (token && 
+        (strncmp(token, "KEY_",4)==0 ||
+         strstr(token, "_KEY_")      || // can be CTRL_ALT_KEY_F3
+	     strncmp(token, "BTN_",4)==0)) {
+        data = token;
         token = strtok_r(NULL," ",&bufPtr);
-        if (token && strncmp(token, "event",5) != 0 ) {
-            isLirc = 0;
+        if (token && strstr(token, "event") == NULL) {  // /dev/input/eventX or eventX
+ 	    isLirc = 0;
         }
     } else {
-        isLirc = 0;
+	isLirc = 0;
     }
 
     if (isLirc) {
