@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2011-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2011-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -23,25 +23,40 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
+#include <fcntl.h>
 
 #include "str.h"
 #include "utils.h"
 #include "conf.h"
 #include "list.h"
 #include "state.h"
+#include "mutex.h"
+#include "gen_xml.h"
 
 extern char tmp[MAXMAXLEN];
 
+boolean_t _stateInited = BOOL_NO;
 static ClientState _state;
 static SingleList* _userMenu = NULL;
 static int         _curForm  = CF;
 
 int _iconPadding   = 0;
 int _iconSize      = 64;
-boolean_t _bottomlineSkin = BOOL_NO;
 
 void initState()
 {
+    INFO2("[DS]: initState");
+    
+    mutexNew(M_STATE);
+    
+
+    _state.cf.layout7x1   = BOOL_NO;
+    _state.cf.upBtn       = NULL;
+    _state.cf.dnBtn       = NULL;
+    _state.cf.joystick    = BOOL_YES;
+    _state.cf.keypad      = BOOL_YES;
+    
     _state.cf.caption     = stringNew("anyRemote");
     _state.cf.title       = stringNew(" ");
     _state.cf.status      = stringNew(" ");
@@ -49,7 +64,8 @@ void initState()
     for (;i<ICON_NUM;i++) {
     	_state.cf.icons[i] = stringNew("default");
     }
-    _state.cf.cover	  = NULL;
+    _state.cf.cover	      = NULL;
+    _state.cf.namedCover  = NULL;
     _state.cf.volume	  = NULL;
     _state.cf.useVolume   = BOOL_NO;
     _state.cf.visual.font = stringNew("small");
@@ -74,6 +90,8 @@ void initState()
     _state.ef.caption     = stringNew("");
     _state.ef.text        = stringNew("");
     _state.ef.label       = stringNew("");
+    
+    _stateInited = BOOL_YES;
 }
 
 static void listItemFree(pointer_t data)
@@ -90,9 +108,21 @@ static void listItemFree(pointer_t data)
 
 void freeState()
 {
+    if (_stateInited == BOOL_NO) {
+        return;
+    }
+    
+    INFO2("[DS]: freeState");
+    
+    mutexLock(M_STATE);
+    
     if (_state.cf.caption) stringFree(_state.cf.caption, BOOL_YES);
-    if (_state.cf.title) stringFree(_state.cf.title, BOOL_YES);
-    if (_state.cf.status) stringFree(_state.cf.status, BOOL_YES);
+    if (_state.cf.title)   stringFree(_state.cf.title,   BOOL_YES);
+    if (_state.cf.status)  stringFree(_state.cf.status,  BOOL_YES);
+    
+    if (_state.cf.upBtn)   stringFree(_state.cf.upBtn,   BOOL_YES);
+    if (_state.cf.dnBtn)   stringFree(_state.cf.dnBtn,   BOOL_YES);
+    
     int i = 0;
     for (;i<ICON_NUM;i++) {
     	if (_state.cf.icons[i]) stringFree(_state.cf.icons[i], BOOL_YES);
@@ -127,6 +157,11 @@ void freeState()
     
     freeMenu();
 
+    mutexUnlock(M_STATE);
+    
+    mutexRemove(M_STATE);
+    
+    _stateInited = BOOL_NO;
 }
 
 // rely on previous strtok()
@@ -167,11 +202,11 @@ void setFgBg(int form, boolean_t set_fg)
     }
     
     if (ptr) {
-	if ((*ptr)) {
+	    if ((*ptr)) {
             (*ptr) = stringNew(buf);
-	} else {
+	    } else {
             stringAssign((*ptr),buf);
-	}
+	    }
     }
 }
 
@@ -189,7 +224,7 @@ void setFont(int form)
     }
     
     string_t** ptr = NULL;
-
+    
     if (form == CF) { 
         ptr = &_state.cf.visual.font;
     } else if (form == TX) {
@@ -199,11 +234,11 @@ void setFont(int form)
     }
     
     if (ptr) {
-	if ((*ptr)) {
+	    if ((*ptr)) {
             (*ptr) = stringNew(s);
-	} else {
+	    } else {
             stringAssign((*ptr),s);
-	}
+	    }
     }
 }
 
@@ -226,25 +261,26 @@ int curForm()
 static void addDefMenu()
 {
     INFO2("[DS]: addDefMenu %d", _curForm);
+    
     switch (_curForm) {
 
-    case TX:
-    case LI:
-        _userMenu = listSingleAppend(_userMenu, stringNew("Back"));
-        break;
+        case TX:
+        case LI:
+            _userMenu = listSingleAppend(_userMenu, stringNew("Back"));
+            break;
 
-    case EF:
-        _userMenu = listSingleAppend(_userMenu, stringNew("Back"));
-        _userMenu = listSingleAppend(_userMenu, stringNew("Ok"));
-        break;
+        case EF:
+            _userMenu = listSingleAppend(_userMenu, stringNew("Back"));
+            _userMenu = listSingleAppend(_userMenu, stringNew("Ok"));
+            break;
 
-    case CF:
-        //_userMenu = listSingleAppend(_userMenu, stringNew("Disconnect"));
-        //_userMenu = listSingleAppend(_userMenu, stringNew("Exit"));
-        break;
+        case CF:
+            //_userMenu = listSingleAppend(_userMenu, stringNew("Disconnect"));
+            //_userMenu = listSingleAppend(_userMenu, stringNew("Exit"));
+            break;
 
-    case WM:
-        break;
+        case WM:
+            break;
     }
 }
 
@@ -263,8 +299,12 @@ void freeMenu()
 void setupDefMenu()
 {
     INFO2("[DS]: setupDefMenu %d", _curForm);
+    mutexLock(M_STATE);
+    
     freeMenu();
     addDefMenu();
+    
+    mutexUnlock(M_STATE);
 }
 
 SingleList* userMenu()
@@ -291,7 +331,7 @@ void setMenu()
     while (isspace(*token)) {
         ++token;
     }
-
+    
     if (strncmp(token,"add",3) == 0 || strncmp(token,"replace",7) == 0) {
 
         logger(L_INF, "[DS]: setMenu add/replace");
@@ -308,17 +348,22 @@ void setMenu()
             }
 
             INFO2("[DS]: Set(menu, ...) %s", token3);
-
+            
+            mutexLock(M_STATE);
+            
             string_t* dup = stringNew(token3);
             _userMenu = listSingleAppend(_userMenu, dup);
             token3 = strtok(NULL,",");
+            
+            mutexUnlock(M_STATE);
         }
     } else if (strncmp(token,"clear",5) == 0) {
+        mutexLock(M_STATE);
         freeMenu();
+        mutexUnlock(M_STATE);
     } else {
         ERROR2("[DS]: Can to parse Set(menu, ...)");
     }
-
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -329,11 +374,15 @@ void setMenu()
 
 void setCfCaption(const char * s)
 {
+    mutexLock(M_STATE);
+    
     if (_state.cf.caption == NULL) {
-        _state.cf.caption =  stringNew(s);
+        _state.cf.caption = stringNew(s);
     } else {
         stringAssign(_state.cf.caption,s);
     }
+    
+    mutexUnlock(M_STATE);
 }
 
 const char* cfCaption()
@@ -345,13 +394,16 @@ void setCfTitle(const char * s)
 {
     boolean_t setupMenu = (_curForm != CF);
 
+    mutexLock(M_STATE);
+    
     if (_state.cf.title == NULL) {
         _state.cf.title =  stringNew(s);
     } else {
         stringAssign(_state.cf.title,s);
     }
+    mutexUnlock(M_STATE);
 
-    _curForm = CF;
+    setCurForm(CF);
 
     if (setupMenu) {
         setupDefMenu();
@@ -367,13 +419,17 @@ void setCfStatus(const char * s)
 {
     boolean_t setupMenu = (_curForm != CF);
 
+    mutexLock(M_STATE);
+    
     if (_state.cf.status == NULL) {
         _state.cf.status =  stringNew(s);
     } else {
         stringAssign(_state.cf.status,s);
     }
 
-    _curForm = CF;
+    mutexUnlock(M_STATE);
+    
+    setCurForm(CF);
 
     if (setupMenu) {
         setupDefMenu();
@@ -396,7 +452,9 @@ void setIcons()
     }
 
     token = strtok(NULL,",");
-
+    
+    mutexLock(M_STATE);
+    
     while (token) {
 
         while (isspace(*token)) {
@@ -419,16 +477,18 @@ void setIcons()
                 ++token;
             }
 
- 	    if (_state.cf.icons[ic] == NULL) {
-        	_state.cf.icons[ic] =  stringNew((token?token:"none"));
-	    } else {
-        	stringAssign(_state.cf.icons[ic],(token?token:"none"));
-	    }
+ 	        if (_state.cf.icons[ic] == NULL) {
+        	    _state.cf.icons[ic] =  stringNew((token?token:"none"));
+	        } else {
+        	    stringAssign(_state.cf.icons[ic],(token?token:"none"));
+	        }
             INFO2("[DS]: setIcons %d %s",ic,_state.cf.icons[ic]->str);
         }
         token = strtok(NULL,",");
     }
-
+    
+    mutexUnlock(M_STATE);
+    
     setCurForm(CF);
 
     if (setupMenu) {
@@ -443,32 +503,76 @@ static void setUseVolume(boolean_t use)
 }
 
 //
-// Set(skin,default|bottomline|3x4|7x1 <--- handle only this for now
+// Set(skin,default|bottomline|3x4|7x1
 //    [,keypad_only|joystick_only]
-//    [,ticker|noticker] [,volume]
-//    [,choose,_button_] [,up,_button_] [,down,_button_])
+//    [,ticker|noticker]   <-- TODO
+//    [,volume]
+//    [,choose,_button_]   <-- TODO
+//    [,up,_button_] [,down,_button_])
 //
 void setSkin()
 {
+    
     setUseVolume(BOOL_NO);  // reset volume usage flag
-    _bottomlineSkin = BOOL_NO;
+    _state.cf.layout7x1 = BOOL_NO;
+    
     char* token;
 
     boolean_t setupMenu = (curForm() != CF);
+    
+    mutexLock(M_STATE);
+    
+    if (_state.cf.upBtn) {
+        stringFree(_state.cf.upBtn, BOOL_YES);
+        _state.cf.upBtn = NULL;
+    }
+    if (_state.cf.dnBtn) {
+        stringFree(_state.cf.dnBtn, BOOL_YES);
+        _state.cf.dnBtn = NULL;
+    }
 
     while((token = strtok(NULL,","))) {
 
-        if (strcmp(token,"bottomline") == 0 || strcmp(token,"7x1") == 0) {
-            _bottomlineSkin = BOOL_YES;
+        if (strcmp(token,"bottomline") == 0 || 
+            strcmp(token,"7x1") == 0 || 
+            strcmp(token,"6x1") == 0 || 
+            strcmp(token,"5x1") == 0) {
+            _state.cf.layout7x1 = BOOL_YES;
         } else if (strcmp(token,"volume") == 0) {
             setUseVolume(BOOL_YES);
+        } else if (strcmp(token,"keypad_only") == 0) {
+            _state.cf.keypad   = BOOL_YES;
+            _state.cf.joystick = BOOL_NO;
+        } else if (strcmp(token,"joystick_only") == 0) {
+            _state.cf.keypad   = BOOL_NO;
+            _state.cf.joystick = BOOL_YES;
+        } else if (strcmp(token,"up") == 0) {
+            token = strtok(NULL,",");
+            if (token) {
+	            if (_state.cf.upBtn == NULL) {
+                    _state.cf.upBtn =  stringNew(token);
+	            } else {
+                    stringAssign(_state.cf.upBtn,token);
+	            }
+            }
+        } else if (strcmp(token,"down") == 0) {
+            token = strtok(NULL,",");
+            if (token) {
+	            if (_state.cf.dnBtn == NULL) {
+                    _state.cf.dnBtn =  stringNew(token);
+	            } else {
+                    stringAssign(_state.cf.dnBtn,token);
+	            }
+            }
         }
     }
 
-    if (_bottomlineSkin) {
-        freeCfCover();
-    }
+    //if (_state.cf.layout7x1) {
+    //    freeCfCover();
+    //}
 
+    mutexUnlock(M_STATE);
+    
     setCurForm(CF);
 
     if (setupMenu) {
@@ -485,15 +589,44 @@ const char* cfIcon(int i)
 
 void setCfCover(const char * s)
 {
+    //printf("setCfCover %s\n", s);
+    mutexLock(M_STATE);
+
     if (s) {
-	if (_state.cf.cover == NULL) {
+	    if (_state.cf.cover == NULL) {
             _state.cf.cover =  stringNew(s);
-	} else {
+	    } else {
             stringAssign(_state.cf.cover,s);
-	}
+	    }
+        if (_state.cf.namedCover) {
+            stringFree(_state.cf.namedCover, BOOL_YES);
+            _state.cf.namedCover = NULL;
+        }
     } else {
         freeCfCover();
     }
+    mutexUnlock(M_STATE);
+}
+
+void setCfNamedCover(char* s) 
+{
+    //printf("setCfNamedCover %s\n", s);
+    mutexLock(M_STATE);
+
+    if (s) {
+	    if (_state.cf.namedCover == NULL) {
+            _state.cf.namedCover =  stringNew(s);
+	    } else {
+            stringAssign(_state.cf.namedCover,s);
+	    }
+        if (_state.cf.cover) {
+            stringFree(_state.cf.cover, BOOL_YES);
+            _state.cf.cover = NULL;
+        }
+    } else {
+        freeCfCover();
+    }
+    mutexUnlock(M_STATE);
 }
 
 const char* cfCover()
@@ -501,9 +634,57 @@ const char* cfCover()
     return (_state.cf.cover ? _state.cf.cover->str : NULL); 
 }
 
+const char* cfNamedCover()
+{
+    return (_state.cf.namedCover ? _state.cf.namedCover->str : NULL); 
+}
+
 void freeCfCover()
 {
-    if (_state.cf.cover) stringFree(_state.cf.cover, BOOL_YES);
+    //printf("freeCfCover\n");
+    if (_state.cf.cover)      stringFree(_state.cf.cover,      BOOL_YES);
+    if (_state.cf.namedCover) stringFree(_state.cf.namedCover, BOOL_YES);
+    _state.cf.cover = NULL; 
+    _state.cf.namedCover = NULL;
+}
+
+// cover-audio -> /usr/share/..../cover-audio.png
+string_t* findNamedCover(const char *name)
+{
+    char *confDir = getCfgDir();
+    string_t* fpath = confDir ? stringNew(confDir) : stringNew("");
+    if (confDir) free(confDir);
+    
+    stringAppend(fpath, "/Icons/common/");
+    stringAppend(fpath, name);
+    stringAppend(fpath, ".png");
+    
+    struct stat statbuf;
+    
+    if (stat(fpath->str, &statbuf) < 0) {  // not found
+
+        char *h = getenv("HOME");
+        if (h) {
+            stringAssign(fpath, h);
+            stringAppend(fpath, "/.anyRemote/Covers/");
+            stringAppend(fpath, name);
+            stringAppend(fpath, ".png");
+            if (stat(fpath->str, &statbuf) < 0) {
+                stringFree(fpath,  BOOL_YES);
+                return NULL;
+            }
+        } else {
+            stringFree(fpath,  BOOL_YES);
+            return NULL;
+        }
+    }
+    
+    if (S_ISREG(statbuf.st_mode) || S_ISLNK(statbuf.st_mode)) {
+        return fpath;
+    }
+    
+    stringFree(fpath,  BOOL_YES);
+    return NULL;
 }
 
 const char* cfBg()
@@ -524,11 +705,11 @@ const char* cfFont()
 void setCfVolume(const char * s)
 {
     if (s) {
-	if (_state.cf.volume == NULL) {
+	    if (_state.cf.volume == NULL) {
             _state.cf.volume =  stringNew(s);
-	} else {
+	    } else {
             stringAssign(_state.cf.volume,s);
-	}
+	    }
     } else {
         setCfVolume("");
     }
@@ -546,7 +727,27 @@ boolean_t useVolume()
 
 boolean_t bottomlineSkin()
 {
-    return _bottomlineSkin;
+    return _state.cf.layout7x1;
+}
+
+const char* cfUpButton()
+{
+    return (_state.cf.upBtn ? _state.cf.upBtn->str : NULL); 
+}
+
+const char* cfDownButton()
+{
+    return (_state.cf.dnBtn ? _state.cf.dnBtn->str : NULL); 
+}
+
+boolean_t useKeypad()
+{
+    return _state.cf.keypad;
+}
+
+boolean_t useJoystick()
+{
+    return _state.cf.joystick;
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -592,18 +793,17 @@ static void setLfList(boolean_t useIcons)
     	if (strlen(token3) > 0) {   // avoid empty list item
     	    char * semicolon = index(token3,':');
     	    if (useIcons && semicolon) {
-    		*semicolon = '\0';
-    		semicolon++;
-    		item->icon   = stringNew(token3);
-    		item->string = stringNew(semicolon);
+    		    *semicolon = '\0';
+    		    semicolon++;
+    		    item->icon   = stringNew(token3);
+    		    item->string = stringNew(semicolon);
     	    } else {
-    		item->icon   = NULL;
-    		item->string = stringNew(token3);
+    		    item->icon   = NULL;
+    		    item->string = stringNew(token3);
     	    }
 
     	    _state.lf.items = listSingleAppend(_state.lf.items, item);
     	}
-
     	token3 = strtok(NULL,",");
     }
 }
@@ -626,8 +826,10 @@ void setList(boolean_t useIcons)
         ++token;
     }
 
+    
     if (strncmp(token,"add",3) == 0 || strncmp(token,"replace",7) == 0) {
 
+        mutexLock(M_STATE);
         _curForm = LI;
 
         char *token2 = strtok(NULL,","); // can be NULL
@@ -645,32 +847,45 @@ void setList(boolean_t useIcons)
             setLfCaption(token2);
         }
         
-	setLfList(useIcons);  // rely on previous strtok()
+	    setLfList(useIcons);  // rely on previous strtok()
+        mutexUnlock(M_STATE);
 
     } else if (strncmp(token,"close",5) == 0) {
        
         char *token2 = strtok(NULL,","); // can be NULL
         if (token2 && strcmp(token,"clear") == 0) {
+            mutexLock(M_STATE);
             freeLfList();
+            mutexUnlock(M_STATE);
         }
         _curForm = CF;
         setupDefMenu();
         return;
     } else if (strncmp(token,"clear",5) == 0) {
+        mutexLock(M_STATE);
         freeLfList();
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"fg",2) == 0) {
+        mutexLock(M_STATE);
         setFgBg(LI,BOOL_YES);
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"bg",2) == 0) {
+        mutexLock(M_STATE);
         setFgBg(LI,BOOL_NO);
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"font",4) == 0) {
+        mutexLock(M_STATE);
         setFont(LI);
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"select",6) == 0) {
+        mutexLock(M_STATE);
         _curForm = LI;
         setLfIndex(strtok(NULL,","));
+        mutexUnlock(M_STATE);
     } else if (strncmp(token,"show",4) == 0) {
         _curForm = LI;
     } else {
@@ -706,9 +921,9 @@ const char* lfFont()
 void freeLfList()
 {
     if (_state.lf.items) {
-	listSingleFullFree(_state.lf.items, listItemFree);
-	_state.lf.items = NULL;
-	_state.lf.selIdx = 1;
+        listSingleFullFree(_state.lf.items, listItemFree);
+        _state.lf.items = NULL;
+        _state.lf.selIdx = 1;
     }
 }
 
@@ -749,7 +964,7 @@ const char* tfCaption()
 
 static void addTfText(const char * s)
 {
-    INFO2("[DS]: addTfText %s", s);
+    //INFO2("[DS]: addTfText %s", s); can be too long
     if (_state.tf.text == NULL) {
         _state.tf.text =  stringNew(s);
     } else {
@@ -797,8 +1012,10 @@ void setText(boolean_t split)
         ++token;
     }
 
+
     if (strncmp(token,"add",3) == 0 || strncmp(token,"replace",7) == 0) {
 
+        mutexLock(M_STATE);
         _curForm = TX;
 
         char *token2 = strtok(NULL,","); // can be NULL
@@ -813,26 +1030,29 @@ void setText(boolean_t split)
         }
 
         if (strcmp(token2,"SAME") != 0) {
-	    setTfCaption(token2);
+	        setTfCaption(token2);
         }
         
-	if (split) {
+	    if (split) {
             char *token3 = strtok(NULL,"\n");
             while (token3) {
 
-        	//INFO2("[DS]: Set(text, ...) %s", token3);
-		addTfText(token3);
-        	token3 = strtok(NULL,"\n");
+        	    //INFO2("[DS]: Set(text, ...) %s", token3);
+		        addTfText(token3);
+        	    token3 = strtok(NULL,"\n");
             }
-	} else {
-	    addTfText(token2 + strlen(token2) + 1);
-	}
+	    } else {
+	        addTfText(token2 + strlen(token2) + 1);
+	    }
+        mutexUnlock(M_STATE);
 	
     } else if (strcmp(token,"close") == 0) {
     
         char *token2 = strtok(NULL,","); // can be NULL
         if (token2 && strcmp(token,"clear") == 0) {
+            mutexLock(M_STATE);
             resetTfText();
+            mutexUnlock(M_STATE);
         }
 	
         _curForm = CF;
@@ -840,16 +1060,24 @@ void setText(boolean_t split)
         return;
 	
     } else if (strncmp(token,"clear",5) == 0) {
+        mutexLock(M_STATE);
         resetTfText();
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"fg",2) == 0) {
+        mutexLock(M_STATE);
         setFgBg(TX,BOOL_YES);
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"bg",2) == 0) {
+        mutexLock(M_STATE);
         setFgBg(TX,BOOL_NO);
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"font",4) == 0) {
+        mutexLock(M_STATE);
         setFont(TX);
+        mutexUnlock(M_STATE);
         return;
     } else if (strncmp(token,"show",4) == 0) {
         _curForm = TX;
@@ -857,6 +1085,7 @@ void setText(boolean_t split)
         ERROR2("[DS]: Can to parse Set(text, ...)");
         return;
     }
+    
     if (setupMenu) {
         setupDefMenu();
     }
@@ -938,13 +1167,18 @@ void setEditfield()
     }
 
     char *token3 = strtok(NULL,",");  // can be NULL
-
+    
+    mutexLock(M_STATE);
+    
     setEfCaption(token1);
     setEfLabel  (token2);
     setEfText   (token3 ? token3 : "");
     setEfPassword(BOOL_NO);
 
-    _curForm = EF;
+    mutexUnlock(M_STATE);
+    
+    setCurForm(EF);
+    
     if (setupMenu) {
         setupDefMenu();
     }
@@ -957,11 +1191,15 @@ void setPassField()
     
     boolean_t setupMenu = (curForm() != EF);
 
+    mutexLock(M_STATE);
+    
     setEfCaption("Enter password");
     setEfLabel  ("Enter password");
     setEfText   ("");
     setEfText   ("");
     setEfPassword(BOOL_YES);
+    
+    mutexUnlock(M_STATE);
 
     setCurForm(EF);
 
@@ -994,6 +1232,7 @@ void setImage(const char* cmd, const char* file)
         ++cmd;
     }
 
+    
     if (strncmp(cmd,"window",6) == 0) {
 
         logger(L_INF, "[DS]: setImage: window");
@@ -1007,12 +1246,14 @@ void setImage(const char* cmd, const char* file)
             ++file;
         }
         
-	if (_state.wf.window) {
-            stringAssign(_state.wf.window, file);
-	} else {
-            _state.wf.window = stringNew(file);
-	}
-
+        mutexLock(M_STATE);
+	    if (_state.wf.window) {
+             stringAssign(_state.wf.window, file);
+	    } else {
+             _state.wf.window = stringNew(file);
+	    }
+        mutexUnlock(M_STATE);
+    
     } else if (strncmp(cmd,"show",4) == 0) {
         setCurForm(WM);
     } else if (strncmp(cmd,"close",5) == 0) {
@@ -1024,6 +1265,7 @@ void setImage(const char* cmd, const char* file)
         return;
     }
 
+    
     if (setupMenu) {
         setupDefMenu();
     }
@@ -1116,3 +1358,115 @@ int iconSize()
 {
     return _iconSize;
 }
+
+static void updateStateSetFile(dMessage* dm)
+{
+    if (strncmp(dm->value,"Set(cover,noname",16) == 0) {
+        setCfCover(dm->file);
+        xmlSetLayoutOk(BOOL_NO);  // Need to regenerate
+    } else if (strncmp(dm->value,"Set(image",9) == 0) {
+        setImage((dm->value + 10), dm->file);  // 10 = 9 + ","
+    } else {
+        ERROR2("[DS]: Unknown command file: %s", (const char*) dm->value);
+    }
+}
+
+static void updateStateSet(dMessage* dm)
+{
+    const char* value = dm->value;
+    
+    DEBUG2("[DS]: updateState %d", (value ? (int) strlen(value) : -1));
+
+    char * cmd = strdup(value);
+
+    stripCommandEnding(cmd);
+
+    if (strlen(cmd) < MAXMAXLEN) {
+        INFO2("[DS]: parse %d %s", curForm(), cmd);
+    } else {
+        INFO2("[DS]: parse %d ... command too long ...", curForm());
+    }
+
+    char* token = strtok(cmd,",");
+    while (isspace(*token)) {
+        ++token;
+    }
+
+    if (strncmp(token,"Set(status",10) == 0) {
+        setCfStatus(cmd+strlen("Set(status,"));
+    } else if (strncmp(token,"Set(title",9) == 0) {
+        setCfTitle(cmd+strlen("Set(title,"));
+    } else if (strncmp(token,"Set(icons",9) == 0) {
+        setIcons();
+        xmlSetLayoutOk(BOOL_NO);  // Need to regenerate
+    } else if (strncmp(token,"Set(font",8) == 0) {
+        mutexLock(M_STATE);
+        setFont(CF);
+        mutexUnlock(M_STATE);
+    } else if (strncmp(token,"Set(fg",6) == 0) {
+        mutexLock(M_STATE);
+        setFgBg(CF,BOOL_YES);
+        mutexUnlock(M_STATE);
+    } else if (strncmp(token,"Set(bg",6) == 0) {
+        mutexLock(M_STATE);
+        setFgBg(CF,BOOL_NO);
+        mutexUnlock(M_STATE);
+    } else if (strncmp(token,"Set(volume",10) == 0) {
+        char* sz = strtok(NULL,",");
+        mutexLock(M_STATE);
+        setCfVolume(sz);
+        mutexUnlock(M_STATE);
+    } else if (strncmp(token,"Set(layout",10) == 0 ||
+               strncmp(token,"Set(skin",8) == 0) {
+        setSkin();
+	    xmlSetLayoutOk(BOOL_NO);  // Need to regenerate
+    } else if (strncmp(token,"Set(cover",9) == 0) {
+        char* subcmd = strtok(NULL,",");
+        if (strncmp(subcmd,"by_name",7) == 0) {
+            setCfNamedCover(subcmd+8);
+        } else if (strncmp(subcmd,"clear",5) == 0) {
+            setCfNamedCover(NULL);
+        } else {
+            ERROR2("[DS]: Improperly formed command Set(cover,...)");
+        }
+    } else if (strncmp(token,"Set(list",8) == 0) {
+        setList(BOOL_NO);
+    } else if (strncmp(token,"Set(iconlist",12) == 0) {
+        setList(BOOL_YES);
+    } else if (strncmp(token,"Set(text",8) == 0) {
+        setText(BOOL_NO);
+    } else if (strncmp(token,"Set(menu",8) == 0) {
+        setMenu();
+    } else if (strncmp(token,"Set(editfield",13) == 0) {
+        setEditfield();
+    } else if (strncmp(token,"Set(image",9) == 0) {
+        ERROR2("[DS]: Improperly formed command Set(image,...)");
+    } else if (strncmp(token,"Set(popup",9) == 0) {
+        // ignore
+    } else if (strncmp(token,"Set(disconnect",14) == 0) {
+        // ignore
+    } else if (strncmp(token,"Set(fullscreen",16) == 0) {
+        // ignore
+    } else if (strncmp(token,"Set(vibrate",11) == 0) {
+        // ignore
+    } else if (strncmp(token,"Set(parameter",13) == 0) {
+        setParams();
+    } else if (strncmp(token,"End",3) == 0) {
+        // ignore
+    } else {
+        ERROR2("[DS]: Unknown command %s", token);
+    }
+
+    free(cmd);
+}
+
+void updateState(dMessage* dm)
+{
+    if (dm->type == DM_SET) {
+        updateStateSet(dm);
+    }
+    if (dm->type == DM_SETFILE) {
+        updateStateSetFile(dm);
+    }
+}
+
