@@ -2,7 +2,7 @@
 // anyRemote
 // a bluetooth remote for your PC.
 //
-// Copyright (C) 2006-2012 Mikhail Fedotov <anyremote@mail.ru>
+// Copyright (C) 2006-2013 Mikhail Fedotov <anyremote@mail.ru>
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -20,18 +20,22 @@
 //
 
 #include <arpa/inet.h>
-#include <stdio.h>
 #include <ctype.h>
+#include <dirent.h>
+#include <errno.h>
+#include <fcntl.h>
+#include <net/if.h>
+#include <netinet/in.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <errno.h>
-#include <time.h>
-#include <sys/wait.h>
+#include <sys/ioctl.h>
+#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <dirent.h>
+#include <sys/wait.h>
+#include <time.h>
+#include <unistd.h>
 
 #include "common.h"
 #include "utils.h"
@@ -696,3 +700,60 @@ string_t* executeDirListCommand(int type, const char* directory)
     }   
     return NULL; 
 }
+
+//////////////////////////////////////////////////////////////////////////////////
+//
+// IP address detection
+//
+//////////////////////////////////////////////////////////////////////////////////
+
+static int get_iface_list(struct ifconf *ifconf)
+{
+   int sock, rval;
+
+   sock = socket(AF_INET,SOCK_STREAM,0);
+   if(sock < 0)
+   {
+       perror("socket");
+       return (-1);
+   }
+
+   if((rval = ioctl(sock, SIOCGIFCONF , (char*) ifconf  )) < 0 ) {
+       perror("ioctl(SIOGIFCONF)");
+   }
+
+   close(sock);
+
+   return rval;
+}
+
+string_t* getLocalIP()
+{
+    struct ifreq ifreqs[20];
+    struct ifconf ifconf;
+
+    memset(&ifconf,0,sizeof(ifconf));
+    ifconf.ifc_buf = (char*) (ifreqs);
+    ifconf.ifc_len = sizeof(ifreqs);
+
+    if(get_iface_list(&ifconf) < 0) {
+        return NULL;
+    }
+    int nifaces =  ifconf.ifc_len/sizeof(struct ifreq);
+
+    DEBUG2("[WS]: getLocalIP found %d interfaces", nifaces);
+
+    int  i;
+    for(i = 0; i < nifaces; i++) {
+        DEBUG2("[WS]: interface #%d is %s", i, ifreqs[i].ifr_name);
+        if (strcmp("lo",ifreqs[i].ifr_name) == 0) {
+            continue;  // skip loopback
+        }
+        if (strcmp("127.0.0.1",inet_ntoa(((struct sockaddr_in *)&ifreqs[i].ifr_addr)->sin_addr)) == 0) {
+            continue;  // skip loopback
+        }
+        DEBUG2("[WS]: use local IP %s", inet_ntoa(((struct sockaddr_in *)&ifreqs[i].ifr_addr)->sin_addr));
+        return stringNew(inet_ntoa(((struct sockaddr_in *)&ifreqs[i].ifr_addr)->sin_addr));
+    }
+    return NULL;
+}
